# engarde_load.tcl - version 1.0.3 - load Engarde data files 
#
# Copyright 2006, Tony Aiuto
#
# Author: Tony Aiuto, tony.aiuto@gmail.com
#
# Public methods:
#	Engarde_LoadCompetition
#	LoadFencersInPoolRound  round#
#	LoadEndOfRound filename VariableToHoldData
#

#
# LoadFencersInPoolRound - read in the claspou_init_N.txt file to get
# the participating fencers in pool round N
# sample:
#q;1;15;
#q;2;12;
#q;3;19;
#q;4;9;
#
proc Engarde_LoadFencersInPoolRound { round } {
  set retval ""
  set fdata [open "claspou_init_$round.txt" ]
  while { ! [eof $fdata] } {
    gets $fdata tmp
    if { [regexp {q;([0-9]*);([0-9]*);} $tmp junk rank fnumber] == 1} {
      lappend retval $fnumber
    }
  }
  close $fdata
  return $retval
}

#
# LoadEndOfRound - read in the claspou_fin or clas_fin_poules file
# to get the ranking at end of a pool round
# sample:
# rank, fencer, bouts, wins, ts, tr
#q;1;8;7;6;34;14;
#q;2;13;7;6;32;24;
#q;3;15;6;5;29;15;
#e;4;15;6;5;29;15;		eliminated fencer
#a;5;0;0;0;0;0;			withdrawn fencer
#
proc LoadEndOfRound { file _retval } {
  global Fencer

  upvar $_retval retval
  set fdata [open $file]
  set idx 0
  while { ! [eof $fdata] } {
    gets $fdata tmp
    if { [regexp {([a-z]);([0-9]*);([0-9]*);([0-9]*);([0-9]*);([0-9]*);([0-9]*);} $tmp junk status rank fnumber bouts wins ts tr] == 1} {
	incr idx
	set retval($idx) [list fnumber $fnumber rank $rank bouts $bouts wins $wins ts $ts tr $tr ]
	# puts "$rank|[FencerName $fnumber]|[FencerClub $fnumber]|$wins/$bouts|[format "%.3f" [expr 1.0 * $wins / $bouts]]|$ts|$tr|[expr $ts - $tr]"

	# stick the rank in the fencer data
	 array set ftmp $Fencer($fnumber)
         set ftmp(pool_rank) $rank
         set ftmp(pool_bouts) $bouts
         set ftmp(pool_wins) $wins
         set ftmp(pool_ts) $ts
         set ftmp(pool_tr) $tr
         set ftmp(pool_status) $status
         set Fencer($fnumber) [array get ftmp]
    }
  }
  close $fdata

  # DEBUG puts "Loaded $idx fencers from $file"
}



#
# LoadEngardeDataFile - load Engarde style data files
#
# sample: {[classe club] [nom "LIFC"] [cle 8]} ...
# 
# loads the array club, indexed by the 'cle' field.  Each element
# is a list that can become an array
#

proc Engarde_LoadEngardeDataFile { filename indexcolumn } {
  set fid [OpenCaseless $filename "r"]
  set fdata [read $fid]
  close $fid

   while { 1 } {
	set element [GetEngardeDataElement fdata]
	if { $element == {} } { break }
	array set x $element
	set classname $x(classe)
	set index $x($indexcolumn)
	global $classname
	set ${classname}($index) $element
	# puts stderr "Class: $classname.  index=$index"
   }
}



#
# pull the next token off a string
#
# input		(ref) input string
# tok		(ref) output token
# type		(ref) token type (WORD, STRING, NUMBER or copy of tok)
#
# Returns:	0 for EOF, 1 for real data
#

set debug_GetToken 0

proc Engarde_GetToken { input _token _type } {
  upvar $input s
  upvar $_token token
  upvar $_type type

  set i 0
  set last [string length $s]
  set ret ""
  set quote ""

  global debug_GetToken
  if { $debug_GetToken } {
   puts "GetToken: [format {%-50.50s} $s]"
  }

  if { $last == 0 } {
    return 0
  }
  # skip leading white space
  for { set i 0 } { $i < $last } { incr i } {
    set c [string index $s $i]

    # skip comments
    if { $c == ";" } {
      while { $i < $last && $c != "\n" } {
	incr i
        set c [string index $s $i]
      }
    }
    if { $c != " " && $c != "\n" && $c != "\t" } {
      break
    }
  }
  # check for end of input
  if { $i >= $last } { return 0 }

  # collect NUMBER
  if { "0" <= $c && $c <= "9" } {
    set ret $c
    incr i
    set type "NUMBER"
    while { $i < $last } {
      set c [string index $s $i]
      if { "0" <= $c && $c <= "9" || $c == "." } {
        append ret $c
        incr i
      } else {
        break
      }
    }
    # reduce the string by the part we parsed off
    set s [string range $s $i end]  
    set token $ret
    return 1
  }

  # collect WORD
  if { [regexp {[a-zA-Z]} $c] == 1} {
    set ret $c
    incr i
    set type "WORD"
    while { $i < $last } {
      set c [string index $s $i]
      if { [regexp {[a-zA-Z0-9_]} $c ] == 1} {
        append ret $c
        incr i
      } else {
        break
      }
    }
    # reduce the string by the part we parsed off
    set s [string range $s $i end]  
    set token $ret
    return 1
  }

  # collect STRING
  if { $c == "\"" } {
    incr i
    set quote $c
    set ret ""
    while { $i < $last } {
      set c [string index $s $i]
      incr i
      if { $quote == $c } {
        # done collecting string
        break
      }
      append ret $c
    }
    # reduce the string by the part we parsed off
    set s [string range $s $i end]  
    set token $ret
    set type "STRING"
    return 1
  }

  if { $c == "(" || $c == ")" || $c == "\{" || $c == "\}" || $c == {[} || $c == {]} } {
    incr i
    # reduce the string by the part we parsed off
    set s [string range $s $i end]  
    set token $c
    set type $c
    return 1
  }
 
  while { $i < $last } {
    set c [string index $s $i]
    if { $c == " " || $c == "\n" || $c == "\t" } {
      break
    } elseif { $c == "(" || $c == ")" || $c == "\{" || $c == "\}" || $c == "\[" || $c == "\]" } {
      # if we have not collected anything, use c as the token
      if { $ret == "" } {
        incr i
        set ret $c
        set type $c
      }
      break
    } else {
      set type "WORD"
      append ret $c
      incr i
    }
  }

  # reduce the string by the part we parsed off
  set s [string range $s $i end]  
  set token $ret
  return 1
}



#
# GetList - parse an Engarde style list of values into a Tcl list
# E.g.
#  nation
#  grille (((*** ()) (d 1) (v 5) (d 1) (d 4) (d 3) (d 2)) ((v 5) (*** ()) (d 4) ...)))
# ({[match (8 nobody () () 8 1 32)]} ...)
#

# internal routine to GetList
set __indent 0
proc pspace { n } {
  for { set i 0 } { $i < $n } { incr i } {
    puts -nonewline " "
  }
}

# internal routine to GetList
proc X_GetList { _value } {
  upvar $_value value

  # global __indent
  # pspace $__indent
  # puts "_GetList( $value )"

  set ret {}
  while { [Engarde_GetToken value tok toktype] } {
    #puts "Got token >>> $tok <<<"
    if { $tok == "(" } {
      # incr __indent
      lappend ret [_GetList value ]
      # incr __indent -1
    } elseif { $tok == ")" } {
      return $ret
    } elseif { $tok == "\{" } {
      lappend ret [GetValue value ]
#OUT } elseif { $tok == "]" || $tok == "\}" } {
#OU    return $ret
    } else {
      lappend ret $tok
    }
  }
  return $ret
}

proc _GetList { _value } {
  upvar $_value value

  set ret {}
  while { 1 } {
    set v [GetValue value]
    if { $v == ")" } {
      break
    }
    lappend ret $v
  }
  return $ret
}

proc YYGetList { value } {
  set tmp $value
  set r [_GetList tmp]
  # we always end up with an extra level
  return [lindex $r 0]	
}

#
# _GetSet - collect a list of value pairs inside a { } wrapper
# sample:
# {[type chaine] [taille 80] [taillecar 12] [si_modifie (verif_club)] ... }

proc _GetSet { _input } {

#puts "GS1"
  upvar $_input input

  set ret ""
  while { [Engarde_GetToken input tok toktype] } {
#puts "GS2 tok=$tok"
    if { $tok == "\}" } {
      break
    }
    if { $tok != {[} } {
      error {_GetSet: expected '['} $tok
    }

    Engarde_GetToken input tok toktype
    if { $toktype != "WORD" } {
      error {_GetSet: expected NAME after [} $tok
    }
    set element $tok
    set value [GetValue input]

#puts "GS3 $element=$value"

    lappend ret [list $element $value]

    Engarde_GetToken input tok toktype
    if { $tok != {]} } {
      error {_GetSet: expected ']'} $tok
    }
  }
  return $ret
}


#
# GetValue - read an Engarde style value
#
# samples
# feminin
# "                  ENGARDE Version 8.0 - 14/9/2002"
# ()
# (marge_gche nom nom_etendu effectif marge_dte)
# {[type chaine] [taille 80] [taillecar 12] [maxcar 12] [si_modifie (verif_club)] [nomchamp
# ({[match (8 nobody () () 8 1 32)]} ...)
#
# When called, we have seen the opening (
#

proc GetValue { _input } {
  upvar $_input input

  # puts "GetValue [string range $input 0 50]"

  if { [Engarde_GetToken input tok toktype] == 0} {
    return ""
  }
  if { $toktype == "WORD"  || $toktype == "STRING"  || $toktype == "NUMBER"} {
    return $tok
  }
  if { $tok == "(" } {
    return [_GetList input ]
  }
  if { $tok == "\{" } {
   return [_GetSet input ]
  }
  return "$tok"
}


#
# GetEngardeDataElement
#
# pick up the next element of the form
# { [classe fooo] [suite a] .... }
#


proc GetEngardeDataElement { _input } {
  upvar $_input input

  #DBG puts "GetEngardeDataElement"
  if { [ Engarde_GetToken input tok toktype] <= 0 } {
    return {}
  }
  if { $tok != "\{" } {
    puts "GetEngardeDataElement: Expected \{, got $tok"
    error "GetEngardeDataElement: Expected \{" $tok
  }

  while { [Engarde_GetToken input tok toktype] } {
    if { $tok == "\}" } {
      break
    }
    if { $tok != {[} } {
      puts "GetEngardeDataElement: expected '\[', got $tok"
      error {GetEngardeDataElement: expected '['} $tok
    }

    Engarde_GetToken input tok toktype
    if { $toktype != "WORD" } {
      error {_GetSet: expected NAME after [} $tok
    }
    set element $tok
    set value [GetValue input]

    set data($element) $value

    Engarde_GetToken input tok toktype
    if { $tok != {]} } {
      puts "GetEngardeDataElement: expected '\]', got $tok"
      error {GetEngardeDataElement: expected ']'} $tok
    }
  }
  # puts stderr "GetEngardeDataElement: returns [array get data]"
  return [array get data]
}



#
# Pool_LoadData - load info for a given pool
#
# Side Effect: Loads Pool array
#
#
#{[numero 1]
# [les_tir_cons (15 3 20 5 2 1 17)]
# [les_tir_feuille (17 15 20 5 3 2 1)]
# [ordre_des_matches odm7]
# [critere_placement nation]
# [grille (((*** ()) (d 1) (v 5) (d 1) (d 4) (d 3) (d 2)) ((v 5) (*** ()) (d 4) (v 5) (v 5)
# (v 5) (v 5)) ((d 4) (v 5) (*** ()) (d 3) (v 5) (d 2) (v 5)) ((v 5) (d 0) (v 5) (***
# ()) (d 2) (d 4) (d 2)) ((v 5) (d 3) (d 2) (v 5) (*** ()) (d 4) (d 3)) ((v 5) (d
# 4) (v 5) (v 5) (v 5) (*** ()) (v 5)) ((v 5) (d 2) (d 1) (v 5) (v 5) (d 4) (*** ())))]
# [piste_no ""]
# [scores ((e 7 17 6 1 16 29) (e 1 15 6 5 29 15) (e 3 20 6 3 24 22) (e 6 5 6 2 18 24) (e 5
# 3 6 2 22 26) (e 2 2 6 5 29 22) (e 4 1 6 3 22 22))]
#}


proc Pool_LoadData { round pool } {
  global Pool

  set filename "pouleT${round}P${pool}.txt"
  set fid [OpenCaseless $filename "r"]
  set fdata [read $fid]
  close $fid

  #DBG puts "loading pool $pool"
  Engarde_GetToken fdata tok toktype
  if { $tok != "\{" } {
    puts "Pool_LoadData: Expected \{, got $tok"
    error "Pool_LoadData: Expected \{" $tok
  }

  while { [Engarde_GetToken fdata tok toktype] } {
    if { $tok == "\}" } {
      break
    }
    if { $tok != {[} } {
      puts "PoolL::LoadData: expected '\[', got $tok"
      error {PoolL::LoadData: expected '['} $tok
    }

    Engarde_GetToken fdata tok toktype
    if { $toktype != "WORD" } {
      error {_GetSet: expected NAME after [} $tok
    }
    set element $tok
    set value [GetValue fdata]

    set data($element) $value

    Engarde_GetToken fdata tok toktype
    if { $tok != {]} } {
      puts "PoolL::LoadData: expected '\]', got $tok"
      error {PoolL::LoadData: expected ']'} $tok
    }
  }
  # parray data

  # copy to global pool vector
  set Pool($data(numero)) [array get data]
}



#;direct elimentation Fence for 3rd
#{[classe suite_tableaux] [critere_constitution classement_initial]
#  [hasard_par_2 non]
#  [protege_debut oui] [qualifies 0] [nom "A"] [nom_etendu "direct elimenation"]
#  [nom_tableaux "table of ^1"] [origine1 "classement_initial 1-1000"]
#  [entites_dans_tableau 10] [cle 1] [nomsy a]
#  [origines ((classement_initial 1 1000))] [nombre_entites 1000]
#  [lestableaux (a1024 a512 a256 a128 a64 a32 a16 a8 a4 a2)] [tableauinitial a16]}
#{[classe suite_tableaux] [critere_constitution progression_naturelle] [hasard_par_2
# non] [protege_debut oui] [qualifies 0] [nom "B"] [nom_etendu "Bout for third place"]
# [nom_tableaux "third place"] [origine1 "battus_tableau A4"] [entites_dans_tableau
# 2] [cle 2] [nomsy b] [origines ((battus_tableau a4))] [nombre_entites 2] [lestableaux
# (b2)]}


#
# Load the tableaux
#
proc LoadAllTableaux { } {
  global DEF_ma_competition
  global table
  global tableSize

  # suite_tableaux is a vector of information about each round of DEs
  # the are named (last_name) A, B, ...
  #
  Engarde_LoadEngardeDataFile "suite_tableaux.txt" "nom"
  # global suite_tableaux
  # parray suite_tableaux

  # loop over the table sets - A is the first round, B is the second ...
  # tableauxactifs is the vector of named tables in use
  foreach tname $DEF_ma_competition(tableauxactifs) {
    set tround [string range $tname 0 0]
    set tsize [string range $tname 1 end]
    if { ! [info exists tableSize($tround)] || $tableSize($tround) < $tsize } {
	set tableSize($tround) $tsize
    }
    set filename "tableau[string toupper $tname].txt"
    # puts "LOAD $filename"
    if { [ file exists $filename ] } {
      set fid [open $filename]
      set tmp [read $fid]
      close $fid
# puts $tmp
# set debug_GetToken 1
      set table($tname) [GetValue tmp]
    }
  }
}


###############################################
#
# routines to load Engarde "def" files
#
# ( def typename   name value name value ... )
#

proc  _Parse_def { _input } {
  upvar $_input s

  #DBG puts "_Parse_def: begin"
  Engarde_GetToken s tok toktype
  if { $tok != "def" } {
    error "_Parse_def: expected 'def'" $tok
  }
  Engarde_GetToken s tok toktype
  if { $toktype != "WORD" } {
    error "_Parse_def: expected def name" $tok
  }
  set myObject "DEF_$tok"
  global $myObject

  #DBG puts "_Parse_def: defining '$myObject'"
 
  set paren 0
  while { [Engarde_GetToken s tok toktype] } {
      if { $tok == ")" } {
	break
      }

      # expecting name
      if { $toktype != "WORD" } {
        error "_Parse_def: expected member name" $tok
      }
      set elementName $tok
      #DBG puts "_Parse_def: got name: $elementName"
      set value [ GetValue s ]
      #DBG puts "_Parse_def: got value: $value"
      set ${myObject}($elementName) $value
  }

#DBG puts "=>>>>============"
#DBG parray $myObject
#DBG puts "=<<<<============"
}

proc LoadDef { filename } {
  global errorInfo
  set fid [open $filename]
  set fdata [read $fid]
  close $fid

  set errorMessage ""
  set errorInfo ""
  catch {
    while { [Engarde_GetToken fdata tok toktype] } {
      # puts "LoadDef:: Got token >>> $tok <<<"

      if { $tok == "" } {
        break
      }
      if { $tok == "(" } {
        _Parse_def fdata
      } else {
        error "LoadDef: expected '('"
      }
    }
  } errorMessage
  if { $errorMessage != "" } {
    puts "Error: $errorMessage"
  }
  if { $errorInfo != "" } {
    puts "next token = $errorInfo"
  }
}

#
# end def file handler
#
#######################################################


#######################################################
#
# Load the engarde competition information and normalize
# it into a structure usable with other products
#

#
# Decode an enumeration using the Engarde built in maps
#
# eg: DEF_competition(sexe) =
#	{type sexe} {taillecar 10}{defaut masculin} {nomchamp sexe}
#	{texte sexe} {enumere {{m masculin male} {f feminin female}}}
#
proc Engarde_decode_enum { value type } {
    global DEF_competition

    set typedef $DEF_competition($type)
    set map [lfind $typedef "enumere"]
    foreach e $map {
        if { [lindex $e 0] == $value || [lindex $e 1] == $value } {
            return [lindex $e 2]
        }
    }
    return $value
}


#
# Clear any intermediate Engarde structures
#
proc Engarde_Clear { } {
    global DEF_ma_formule
    if { [info exists DEF_ma_formule] } { unset DEF_ma_formule }
    global Competition
    if { [info exists Competition] } { unset Competition }
    global Fencer
    if { [info exists Fencer] } { unset Fencer }
    global DEF_ma_competition
    if { [info exists DEF_ma_competition] } { unset DEF_ma_competition }
    global Club
    if { [info exists Club] } { unset Club }
}


#
# Normalize Club data
#

proc Engarde_NormalizeClubData { } {
    global club		# source
    global Club		# destination

    # parray club
    foreach i [lsort -integer [array names club]] {
	array set x $club($i)
	if { ![ info exists x(nom)] } continue

	set name $x(nom)
	set nation ""
	if { [info exists x(nation1)] } { set nation $x(nation1) }
	set FRED_id "-$i"
	if { [info exists x(FRED_id)] } { set FRED_id $x(FRED_id) }
	set Club($i) [list index $i \
			name $name \
			nation $nation \
			FRED_id $FRED_id \
			]
	unset x
    }
    # parray Club
    foreach i [array names club]  { unset club($i) }
}

#
# Engarde_LoadCompetition 
#
# Load the basic tournement data
#
#

proc Engarde_LoadCompetition { } {
    global Competition Club Fencer
    global tireur
    global DEF_ma_formule

    Engarde_LoadEngardeDataFile "tireur.txt" "cle"
    #parray tireur
    if { [file exists "club.txt"] } {
      Engarde_LoadEngardeDataFile "club.txt" "cle"
    } elseif { [file exists "CLUB.TXT"] } {
      Engarde_LoadEngardeDataFile "CLUB.TXT" "cle"
    } else {
      puts "I cannot find the club database: club.txt"
      exit 1
    }
    Engarde_LoadEngardeDataFile "tour_de_poules.txt" "cle"
    #parray tour_de_poules
    LoadDef "competition.egw"

    #
    # Normalize into our data structure
    #
    Engarde_NormalizeClubData

    # Normalize Fencer data
    #
    set Competition(nFencers) 0
    foreach i [lsort -integer [array names tireur]] {

	array set x $tireur($i)

	# Engarge reports data at periodic times even for absent fencers
	# you must track that and skip them in the printing
	# we also only want to count the present fenceers
	set isPresent 1
	if { [info exists x(presence)] && $x(presence) != "present"} {
		set isPresent 0
	} else {
		incr Competition(nFencers)
	}
	# sometimes they drop out
	if { [info exists x(status)] && $x(status) == "abandon"} {
		set isPresent 0
	}

	# This is all full of hackery....
	# Many USFA clubs put the USFA classification in the license
	# field.  Sometimes they put it in the license_fie field
	# The LIUSFA sometimes really botches up and puts the
	# class in license_fie, THEN GOES BACK AT THE END OF THE
	# EVENT an puts the Qualifier # in license and the change
	# in class in license_fie.   AGGGGH!

	set rank ""
	if { [info exists x(licence) ] } {
		set tmp [GetUSFAClass $x(licence)]
		if { $tmp != "" } {
			set rank $tmp
			# do not use it again
			unset x(licence)
		}
	}
	if { $rank == "" && [info exists x(licence_fie) ] } {
		set rank [GetUSFAClass $x(licence_fie)]
	}


	# LIUSFA hackery - using mode column for auto qualifiers
	set autoqual ""
	if { [info exists x(mode)] } {
		# puts "$x(nom) is mode $x(mode)<br/>"
		set autoqual $x(mode)
		unset x(mode)
	}

	# LIUSFA hackery - and sometimes they stick it in licence - no love
	# for consistency there.
	if { [info exists x(licence) ] && [regexp -nocase "auto" $x(licence)] } {
		set autoqual auto
		unset x(licence)
	}

	set birth_year ""
	if { [info exists x(date_nais)] } { set birth_year $x(date_nais) }
	set club ""
	if { [info exists x(club1)] } { set club $x(club1) }
	set FRED_id "-$x(cle)"
	if { [info exists x(FRED_id)] } { set FRED_id $x(FRED_id) }

	set Fencer($i) [list cle $x(cle) index $x(cle) \
			last_name $x(nom) \
			first_name $x(prenom) \
			birth_year $birth_year \
			club $club \
			usfa_rank $rank \
			final_usfa_rank $rank \
			final_rank 99999 \
			seed 99999 \
			isPresent $isPresent \
			FRED_id $FRED_id \
			auto_qual $autoqual ]
	unset x
    }
    # we do not need this any more
    unset tireur

    global DEF_ma_competition

    if { [info exists DEF_ma_competition(FRED_id)] } {
	set Competition(FRED_id) $DEF_ma_competition(FRED_id)
    } else {
	set Competition(FRED_id) "-1"
    }

    set Competition(sex) [Engarde_decode_enum $DEF_ma_competition(sexe) sexe]
    if { [info exists DEF_ma_competition(categorie)] } {
        set Competition(category) [Engarde_decode_enum \
				$DEF_ma_competition(categorie) categorie]
    } else {
        set Competition(category) "senior"
    }

    set Competition(date) $DEF_ma_competition(date)
    set Competition(title1) $DEF_ma_competition(titre1)
    if { [info exists DEF_ma_competition(titre2)] } {
      set Competition(title2) $DEF_ma_competition(titre2)
    } else {
      set Competition(title2) ""
    }
    if { [info exists DEF_ma_competition(titre3)] } {
      set Competition(title3) $DEF_ma_competition(titre3)
    } else {
      set Competition(title3) ""
    }
    if { [info exists DEF_ma_competition(titre4)] } {
      set Competition(title4) $DEF_ma_competition(titre4)
    } else {
      set Competition(title4) ""
    }
    if { [info exists DEF_ma_competition(federation)] } {
      set Competition(federation) [string toupper $DEF_ma_competition(federation)]
    } else {
      set Competition(federation) "USFA"
    }
    if { [info exists DEF_ma_competition(organisateur)] } {
      set Competition(organizer) $DEF_ma_competition(organisateur)
    }
    set Competition(weapon) [Engarde_decode_enum $DEF_ma_competition(arme) arme]
    set Competition(title_short) $DEF_ma_competition(titre_reduit)
    set Competition(title_long) $DEF_ma_competition(titre_ligne)

    set Competition(third_place_table) {}
    if { [info exists DEF_ma_competition(troisieme_place)] } {
	set Competition(third_place_table) $DEF_ma_competition(troisieme_place)
    }

    if { [info exists DEF_ma_formule(nutour)] } {
	set Competition(nPoolRounds) $DEF_ma_formule(nutour)
    } else {
	set Competition(nPoolRounds) 1
    }

    set Competition(phase) ""
    if { [info exist DEF_ma_formule(etat)] } {
	if { $DEF_ma_formule(etat) == "debut" } {
	    set Competition(phase) "debut"
	} elseif { $DEF_ma_formule(etat) == "poules" } {
	    set Competition(phase) "pools"
	} elseif { $DEF_ma_formule(etat) == "en_cours" } {
	    set Competition(phase) "in progress"
	} elseif { $DEF_ma_formule(etat) == "termine" } {
	    set Competition(phase) "complete"
	}
    }

#  domaine_compe national
#  type_compe individuelle
#  version "                ENGARDE Version 8.16 - 11/11/2005"
#  version_fic 80
#  titre_reduit "Women Sabre Div II III Summer '06"
#  titre_ligne "Women Sabre Div II III Summer National '06"
#  troisieme_place b2
#  num_svg 3
#  tableauxactifs (a16 a8 a4 a2 b2)
#  lestableaux (a1024 a512 a256 a128 a64 a32 a16 a8 a4 a2 b2)
#  tableaux_en_cours (b2)

}

#
# Done loading the basic tournement data
#
###############################################

# 
# Engarde_LoadPoolRound - load the data for a given round of pools
#
# tour_de_pools is indexed by round and tells us overall data
# the files pouleTrPn.txt are the data for round r, pool n.
#

proc Engarde_LoadPoolRound { round } {
	global tour_de_poules PrintStyle
	global FLIST
	global RoundInfo

	set FLIST($round) [Engarde_LoadFencersInPoolRound $round]

	# load and print the individual pools
	array set tmpRoundInfo $tour_de_poules($round)
	for { set pool 1 } { $pool <= $tmpRoundInfo(nombre_poules) } { incr pool } {
		global Pool

		# puts "Call Pool_LoadData $round $pool"
		Pool_LoadData $round $pool
	}
	set rdata(fencers_in) $tmpRoundInfo(entites_dans_poules)
	set rdata(fencers_out) $tmpRoundInfo(qualifies)
	set rdata(nTouches) $tmpRoundInfo(nombre_touches_poules)
	set RoundInfo($round) [array get rdata]
#puts "LoadPoolRound: FLIST length=[llength $FLIST($round)]: $FLIST($round)"
}


###############################################
#
# Load all pool available pool data for an Engarde Competition
#

proc Engarde_LoadPools { } {
    global Fencer Competition

    if { $Competition(phase) == "debut" } return

    for { set round 1 } { $round <= $Competition(nPoolRounds) } { incr round } {
      set filestring "claspou_init_$round.txt"
      if { [file exists $filestring] } {
	Engarde_LoadPoolRound $round
      }

#OUT      if { $round <= $Competition(nPoolRounds) } {
        if { [file exists "claspou_fin_$round.txt"] } {
          global EndOfRoundData_$round
          LoadEndOfRound "claspou_fin_$round.txt" EndOfRoundData_$round
        }
#OUT      }
    }

    if { [file exists "clas_fin_poules.txt"] } {
        global EndOfPoolsData
	LoadEndOfRound "clas_fin_poules.txt" EndOfPoolsData
    }
}


proc Engarde_LoadTables { } {
    global Competition Fencer

    #
    # if we are still in the pools, stop here
    #
    if { $Competition(phase) == "pools" } { return 0 }

    # if we finished pools, but have not started DEs
    global DEF_ma_competition
    if { ! [info exists DEF_ma_competition(tableauxactifs)] } {
       return 0
    }

    #
    # calculate seeds for DEs
    # also figure out where you would be in the printed tables
    set bouts [GenerateDEBouts $Competition(nFencers)]
    SeedToDETable $bouts seed2table
    global EndOfPoolsData
    foreach fi [lsort -integer [array names EndOfPoolsData]] {
	array set fd $EndOfPoolsData($fi)
	set FI $fd(fnumber)
	# puts $out "<br/>$fi - $fd(rank) - [FencerName $FI]"
	array set tmp $Fencer($FI)
	set tmp(seed) $fd(rank)
	set tmp(final_rank) $fd(rank)
	set tmp(tableIndex) $seed2table($fd(rank))
	set Fencer($FI) [array get tmp]
    }

    LoadAllTableaux
    return 1
}

#
#
###############################################


proc Engarde_load_test { } {
	Engarde_LoadCompetition
}

