# fencing_data.tcl - routines to deal with fencing data
#
# Copyright 2006, Tony Aiuto
#
# Author: Tony Aiuto, tony.aiuto@gmail.com
#

proc clearGlobal { varname } {

	global $varname

	if { [info exists $varname] } {
		foreach f [array names $varname] {
			set foo "$varname"
			append foo {(} $f {)}
	    		unset $foo
		}
		unset $varname
	}
}


proc InitCompetition { } {
    global Competition

    clearGlobal Competition
    clearGlobal FLIST
    clearGlobal Fencer
    clearGlobal Award
    clearGlobal EndOfPoolsData
    clearGlobal tableSize
    clearGlobal suite_tableaux
    clearGlobal RoundInfo


    set Competition(year) [clock format [clock seconds] -format "%y"]

    set Competition(classification) "NR"
    set Competition(isQualifier) ""
    set Competition(minQualify) 3
    set Competition(haveFinal) 0

#   set Fencer(nobody) [list index nobody last_name "" first_name "" \
#			birth_year 0 club "" usfa_rank "U" ]


    # hook to local override function
    if { [info procs FixupThisComp] == "FixupThisComp" } {
    	FixupThisComp
    }
}


proc _FencerNameCompare { a b } {
  global Fencer
  array set a1 $Fencer($a)
  array set b1 $Fencer($b)
  set ret [string compare $a1(last_name) $b1(last_name)]
  if { $ret != 0 } { return $ret }
  return [string compare $a1(first_name) $b1(first_name)]
}

proc _FencerSeedCompare { a b } {
  global Fencer

  if { $a == "nobody" } {
    if { $b == "nobody" } { return 0 } else { return -1 }
  } elseif { $b == "nobody" } {
    return 1
  }
  array set a1 $Fencer($a)
  array set b1 $Fencer($b)
  set ret [expr $a1(seed) - $b1(seed)]
  if { $ret != 0 } { return $ret }
  return [string compare $a1(last_name) $b1(last_name)]
}

proc _FencerRankCompare { a b } {
  global Fencer
  array set a1 $Fencer($a)
  array set b1 $Fencer($b)
    set arank $a1(final_rank)
    set brank $b1(final_rank)
  set ret [expr $arank - $brank ]
  if { $ret != 0 } { return $ret }
  return [string compare $a1(last_name) $b1(last_name)]
}


proc FencersByName { } {
   global Fencer
   return [lsort -command _FencerNameCompare [array names Fencer]]
}

proc FencersByRank { } {
   global Fencer
   return [lsort -command _FencerRankCompare [array names Fencer]]
}





proc ClubName { n } {
  global Club
  if { $n == "" } return ""
  array set x $Club($n)
  return $x(name)
}

proc FencerName { n } {
  global Fencer
  if { $n == "nobody" } {
    return ""
  }
  array set x $Fencer($n)
  return "$x(last_name), $x(first_name)"
}

proc FencerLastname { n } {
  global Fencer
  if { $n == "nobody" } {
    return ""
  }
  array set x $Fencer($n)
  return "$x(last_name)"
}

proc FencerClub { n } {
  global Fencer
  array set x $Fencer($n)
  return [ClubName $x(club)]
}

proc USFARank { _finfo } {
  upvar $_finfo finfo
  if { [info exists finfo(licence) ] } {
    return "$finfo(licence)"
  } elseif { [info exists finfo(licence_fie) ] } {
    return $finfo(licence_fie)
  } else {
    return ""
  }
}



#
# RankFencers - set current rankings for a group of fencers
#
# Sort them by initial seed and assign ranks from a given spot
#
# flist - list of fencer #s
# startrank - first rank to assign
#

proc RankFencers { flist start_rank } {
    global Competition Fencer

    # cur_rank is rank to assign
    set cur_rank $start_rank
    # i is rank based on absolute index
    set i $cur_rank
    # oseed is the seed of the last fencer (to look for ties)
    set oseed ""
    foreach FI [lsort -command _FencerSeedCompare $flist ] {
      array set x $Fencer($FI)
      # puts "<br/>$i [FencerName $FI]"
      # check for ties in initial seeding
      if { $x(seed) != $oseed } {
	set cur_rank $i
        set oseed $x(seed)
      }

      # This is a hack to get 3rd and 4th to be a tie.
      # We key on the presence of a third place table.  If there
      # is NOT one, and you are 4th.   We just make you 3rd.
      if { $cur_rank == 4 && $Competition(third_place_table) == "" } {
	set cur_rank 3
      }

      # Alternate hack.  This one relies on us being in
      # a table that is based on 1 (like a4), rather than
      # the one which fences off 3rd and 4th (b2)
      # if { $cur_rank == 4 && $start_rank == 1 } { set cur_rank 3 }

      set x(final_rank) $cur_rank
      set Fencer($FI) [array get x]
      incr i
    }
}



#
# Given an ordered set of seeds in bouts at a table size,
# expand to the next size
# E.g.   1 2 		->  1 4  3 2
# E.g.   1 4  3 2	->  1 8  5 4  3 6  7 2
#

proc FencingData_ExpandBoutList { boutList } {
	set tableSize [llength $boutList]
	if { $tableSize == 0 } { return [list 1] }

	set ret {}
	# LR is 0 for LEFT, 1 for RIGHT
	set LR 0
	incr tableSize $tableSize
	foreach seed $boutList {
		set opponent [expr $tableSize + 1 - $seed]

		if { $LR == 0 } {
			lappend ret $seed $opponent
		} else {
			lappend ret $opponent $seed
		}
		set LR [expr 1 - $LR ]
	}
	return $ret
}

#
# Generated an ordered set of seeds representing the opponents
# in bouts at a given table size
#

proc GenerateDEBouts { tableSize } {

	# make sure tableSize is a power of 2
	set depth 0
	set ts 1
	while { $ts < $tableSize } {
		incr depth 1
		incr ts $ts
	}

	set bouts {}
	for { set i $depth } { $i >= 0 } { incr i -1 } {
		set bouts [FencingData_ExpandBoutList $bouts ]
	}
	return $bouts
}


#
# Using a list of seeds paired into bouts, figure out where you
# would show up in the DE tables.
#
#

proc SeedToDETable { boutList _seedToDE } {
	upvar $_seedToDE seedToDE

	set page 0
	set line 0
	set i 0
	foreach seed $boutList {
		# only 32 names are on a page
		if { $i % 32 == 0 } {
			incr page
			set line 0
		}
		incr i
		incr line

		#puts "$seed   $line $page"
		set seedToDE($seed) [list $page $line]
	}
}


#
# FD_CalculateRanksAfterDERound
#
# Calculate current rank of all fencers after a round
# The basic algorithm is to
#	walk through a round of DE's
#	seperate fencers into a list of winners and losers
#	sort each group by initial seed (after bools)
#	fill the sorted winners into slots 1..n and the
#	  losers into n+1..2n
#	Except it's a little trickier because in a B round
#	we are filling people into other locations than 1..2n
#

proc FD_CalculateRanksAfterDERound { round } {
  global Competition Fencer
  global table
  global tableSize

  # puts "T size = $tableSize($round)"

    # The winners and losers from this round fill a certain set of
    # slots in the final rankings.  For the A round, that is
    # the normal positions.  For a B round, we are back filling
    # slots.   A B2 round should be for 3rd place, so I think
    # the right thing is ...
    # for Bn round, the slots are n+1...n+n
    set roundBase 1
    if { $round == "a" } {
	set roundBase 1
    } elseif { $round == "b" } {
    	set roundBase 3
    } elseif { $round == "c" } {
	# wild guess 2^2+1, based on a = 2^0 + 1, b=2^1+1
        set roundBase 5
    }

  #
  # roll down the tables from largest to smallest and get the match
  # data for each.    Transpose that into a vector (by column) of
  # who where the fencers and the # of points they got
  set col 0
  for { set tsize $tableSize($round) } { $tsize > 1 } { set tsize [expr $tsize / 2]} {
    # puts $table($round$tsize)
    set fl ""
    set sl ""
    set seeds ""
    set wlist {}
    set llist {}

    if { [ info exists table($round$tsize) ] } {
      set matches [lfind $table($round$tsize) les_matches]

      foreach m $matches {
    	set match [lfind $m match]
	# break out the match data
	set fencer1 [lindex $match 0]
	set fencer2 [lindex $match 1]
	set points1 [lindex $match 2]
	set points2 [lindex $match 3]
	set winner  [lindex $match 4]
	set seed1 [lindex $match 5]
	set seed2 [lindex $match 6]
	lappend fl $fencer1 $fencer2
	lappend sl $points1 $points2
	lappend seeds $seed1 $seed2

	#
	# keep seeds up to date
	# we make a list of winners and losers and sort by initial seed
	# at the end of the round

	if { $winner == $fencer1 } {
	  if { $fencer1 != "" } { lappend wlist $fencer1 }
	  if { $fencer2 != "nobody" && $fencer2 != ""} {
	    lappend llist $fencer2
	  }
	} elseif { $winner == $fencer2 } {
	  # fencer 2 won the bout
          if { $fencer2 != "" } { lappend wlist $fencer2 }
	  if { $fencer1 != "nobody" && $fencer1 != ""} {
	    lappend llist $fencer1
	  }
	} else {
	  # bout has not happend yet
	  if { $seed1 < $seed2 } {
	    lappend wlist $fencer1
	    lappend llist $fencer2
	  } else {
	    lappend llist $fencer1
	    lappend wlist $fencer2
	  }
        }
      }
    } else {
      for { set i 0 } { $i < $tsize } { incr i } {
	lappend fl unset
	lappend sl {}
	lappend seeds {}
      }
      set winner unset
    }

    incr col

    # Adjust the current rankings based on the winners and losers
    # of this round

    set i $roundBase
    # puts "<br/>winners: start rank at $i: $wlist"
    RankFencers $wlist $i
    incr i [llength $wlist]
    unset wlist
    # puts "<br/>losers: start rank at $i: $llist"
    RankFencers $llist $i
    unset llist
  }
}


#
# Calculate the final ranks for each fencer
# We do that by waking through each DE round and re-ranking
# after each until we are done
#

proc CalculateFinalRanks { } {
  global tableSize
  global suite_tableaux

  foreach round [list a b c d] {
    if { [info exists tableSize($round)] } {
      array set rdata $suite_tableaux([string toupper $round])
      #  puts "$rdata(nom_tableaux)"
      FD_CalculateRanksAfterDERound $round
    }
  }
}

